IDI_ICON1               ICON    DISCARDABLE     "qnapi.ico"

#include <windows.h>

VS_VERSION_INFO VERSIONINFO
FILEVERSION     0,2,3,0
PRODUCTVERSION  0,2,3,0

BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "FileDescription",    "Aplikacja do pobierania napisow do filmow"
            VALUE "FileVersion",        "0.2.3"
            VALUE "InternalName",       "QNapi"
            VALUE "OriginalFilename",   "qnapi.exe"
            VALUE "ProductName",        "QNapi"
            VALUE "ProductVersion",     "0.2.3"
        END
    END

    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0415, 1200
    END
END
